@@ -86,14 +86,18 @@ static guint statfs_id = 0;
 
 typedef struct {
         gboolean notified;
-        double last_pct;
+        double last_notified;
 	char *udi;
 } statfs_mount_info;
 
+#ifdef ENABLE_NOTIFY
 static void statfs_mount_info_add (const char *udi);
 static void statfs_mount_info_free (statfs_mount_info *info);
 static gboolean gvm_statfs_check_space (const char *udi, statfs_mount_info *info, gpointer user_data);
+#endif
 
+static gboolean gvm_dbus_init (void);
+static LibHalContext *gvm_hal_init (void);
 
 static gboolean gvm_user_at_console (void);
 static gboolean gvm_user_active_at_console (void);
@@ -173,7 +177,6 @@ static struct {
 	{ GCONF_ROOT "autotablet",                TYPE_BOOL,   &config.autotablet                },
 	{ GCONF_ROOT "autotablet_command",        TYPE_STRING, &config.autotablet_command        },
 	{ GCONF_ROOT "percent_threshold",         TYPE_FLOAT,  &config.percent_threshold         },
-	{ GCONF_ROOT "percent_freed",             TYPE_FLOAT,  &config.percent_freed             },
 	{ GCONF_ROOT "percent_used",              TYPE_FLOAT,  &config.percent_used              },
 };
 
@@ -198,41 +201,41 @@ enum {
 };
 
 static struct _GvmPromptButton GVM_BUTTONS_AUTORUN[] = {
-	{ N_("_Ignore"), NULL, GTK_RESPONSE_CANCEL },
+	{ N_("Ig_nore"), NULL, GTK_RESPONSE_CANCEL },
 	{ N_("_Allow Auto-Run"), NULL, GVM_RESPONSE_RUN },
 };
 
 static struct _GvmPromptButton GVM_BUTTONS_CAMERA[] = {
-	{ N_("_Ignore"), NULL, GTK_RESPONSE_CANCEL },
+	{ N_("Ig_nore"), NULL, GTK_RESPONSE_CANCEL },
 	{ N_("Import _Photos"), NULL, GVM_RESPONSE_IMPORT_PHOTOS },
 };
 
 static struct _GvmPromptButton GVM_BUTTONS_STORAGE_CAMERA[] = {
-	{ N_("_Ignore"), NULL, GTK_RESPONSE_CANCEL },
+	{ N_("Ig_nore"), NULL, GTK_RESPONSE_CANCEL },
 	{ N_("_Open Folder"), NULL, GVM_RESPONSE_BROWSE },
 	{ N_("Import _Photos"), NULL, GVM_RESPONSE_IMPORT_PHOTOS },
 };
 
 static struct _GvmPromptButton GVM_BUTTONS_IPOD_PHOTO[] = {
-	{ N_("_Ignore"), NULL, GTK_RESPONSE_CANCEL },
+	{ N_("Ig_nore"), NULL, GTK_RESPONSE_CANCEL },
 	{ N_("Import _Photos"), NULL, GVM_RESPONSE_IMPORT_PHOTOS },
 	{ N_("Manage _Music"), NULL, GVM_RESPONSE_SYNC_MUSIC },
 };
 
 static struct _GvmPromptButton GVM_BUTTONS_MIXED_CD[] = {
-	{ N_("_Ignore"), NULL, GTK_RESPONSE_CANCEL },
+	{ N_("Ig_nore"), NULL, GTK_RESPONSE_CANCEL },
 	{ N_("_Browse Files"), NULL, GVM_RESPONSE_BROWSE },
 	{ N_("_Play CD"), NULL, GVM_RESPONSE_PLAY },
 };
 
 static struct _GvmPromptButton GVM_BUTTONS_BURN_CD[] = {
-	{ N_("_Ignore"), NULL, GTK_RESPONSE_CANCEL },
+	{ N_("Ig_nore"), NULL, GTK_RESPONSE_CANCEL },
 	{ N_("Make _Audio CD"), NULL, GVM_RESPONSE_BURN_AUDIO_CD },
 	{ N_("Make _Data CD"), NULL, GVM_RESPONSE_BURN_DATA_CD },
 };
 
 static struct _GvmPromptButton GVM_BUTTONS_BURN_DVD[] = {
-	{ N_("_Ignore"), NULL, GTK_RESPONSE_CANCEL },
+	{ N_("Ig_nore"), NULL, GTK_RESPONSE_CANCEL },
 	{ N_("Make _DVD"), NULL, GVM_RESPONSE_BURN_DATA_CD },
 };
 
@@ -1999,34 +2002,6 @@ gvm_media_changed (const char *udi, const char *storage_device)
 }
 
 
-/* Return whether or not the device should be automounted based on the
- * HAL automount hint. Default to returning TRUE if the property isn't
- * set for the device or its parent storage device. */
-static gboolean
-gvm_automount_enabled (const char *udi)
-{
-	gboolean enabled = TRUE;
-	DBusError error;
-	char *drive;
-	
-	if (libhal_device_property_exists (hal_ctx, udi, "storage.automount_enabled_hint", NULL))
-		return libhal_device_get_property_bool (hal_ctx, udi, "storage.automount_enabled_hint", NULL);
-	
-	dbus_error_init (&error);
-	if (!(drive = libhal_device_get_property_string (hal_ctx, udi, "info.parent", &error))) {
-		warn ("couldn't get info.parent for %s: %s", udi, error.message);
-		dbus_error_free (&error);
-		return TRUE; /* sensible default? */
-	}
-	
-	if (libhal_device_property_exists (hal_ctx, drive, "storage.automount_enabled_hint", NULL))
-		enabled = libhal_device_get_property_bool (hal_ctx, drive, "storage.automount_enabled_hint", NULL);
-	
-	libhal_free_string (drive);
-	
-	return enabled;
-}
-
 static void
 import_camera_cb (GvmPromptCtx *ctx, int action)
 {
@@ -2050,9 +2025,12 @@ block_device_added (const char *udi, const char *capability __attribute__((__unu
 	char *fsusage = NULL, *device = NULL, *storage_device = NULL;
 	DBusError error;
 	int mountable;
+	int crypto;
 	
 	dbus_error_init (&error);
 	
+	crypto = FALSE;
+	
 	/* is this a mountable volume? */
 	if (!(mountable = libhal_device_get_property_bool (hal_ctx, udi, "block.is_volume", NULL))) {
 		dbg ("not a mountable volume: %s\n", udi);
@@ -2074,6 +2052,7 @@ block_device_added (const char *udi, const char *capability __attribute__((__unu
 			dbg ("encrypted volume found: %s\n", udi);
 			/* TODO: handle encrypted volumes */
 			mountable = FALSE;
+			crypto = TRUE;
 		} else if (strcmp (fsusage, "filesystem") != 0) {
 			dbg ("no sensible filesystem for %s\n", udi);
 			mountable = FALSE;
@@ -2097,12 +2076,12 @@ block_device_added (const char *udi, const char *capability __attribute__((__unu
 			goto out;
 	}
 	
-	if (config.automount_drives && mountable) {
+	if (config.automount_drives && (mountable || crypto)) {
 		if (!gvm_udi_is_subfs_mount (udi)) {
-			if (gvm_automount_enabled (udi)) {
+			if (!libhal_device_get_property_bool (hal_ctx, udi, "volume.ignore", NULL)) {
 				gvm_device_mount (udi, TRUE);
 			} else {
-				dbg ("storage.automount_enabled_hint set to false on %s, not mounting\n", udi);
+				dbg ("volume.ignore set to true on %s, not mounting\n", udi);
 			}
 		}
 	}
@@ -2523,52 +2502,83 @@ hal_device_condition (LibHalContext *ctx __attribute__((__unused__)),
 		gvm_device_eject (udi);
 }
 
-/** Integrate a dbus mainloop. 
- *
- *  @param  ctx                 LibHal context
- *  @param  error     		pointer to a D-BUS error object
- *
- *  @return 			TRUE if we connected to the bus
- */
-static dbus_bool_t 
-hal_mainloop_integration (LibHalContext *ctx, DBusError *error)
+static gboolean
+reinit_dbus (gpointer user_data)
 {
-	dbus_connection = dbus_bus_get (DBUS_BUS_SYSTEM, error);
-	
-	if (dbus_error_is_set (error))
+	if (gvm_dbus_init ()) {
+		if ((hal_ctx = gvm_hal_init ()))
+			libhal_ctx_set_dbus_connection (hal_ctx, dbus_connection);
+		else
+			exit (1);
+		
 		return FALSE;
-	
-        dbus_connection_setup_with_g_main (dbus_connection, NULL);
-	
-	libhal_ctx_set_dbus_connection (ctx, dbus_connection);
+	}
 	
 	return TRUE;
 }
 
+static DBusHandlerResult
+gvm_dbus_filter_function (DBusConnection *connection, DBusMessage *message, void *user_data)
+{
+	if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected") &&
+	    strcmp (dbus_message_get_path (message), DBUS_PATH_LOCAL) == 0) {
+		libhal_ctx_free (hal_ctx);
+		hal_ctx = NULL;
+		
+		dbus_connection_unref (dbus_connection);
+		dbus_connection = NULL;
+		
+		g_timeout_add (3000, reinit_dbus, NULL);
+		
+		return DBUS_HANDLER_RESULT_HANDLED;
+	}
+	
+	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
+static gboolean
+gvm_dbus_init (void)
+{
+	DBusError error;
+	
+	dbus_error_init (&error);
+	if (!(dbus_connection = dbus_bus_get (DBUS_BUS_SYSTEM, &error))) {
+		dbg ("could not get system bus: %s\n", error.message);
+		dbus_error_free (&error);
+		return FALSE;
+	}
+	
+	dbus_connection_setup_with_g_main (dbus_connection, NULL);
+	dbus_connection_set_exit_on_disconnect (dbus_connection, FALSE);
+	
+	dbus_connection_add_filter (dbus_connection, gvm_dbus_filter_function, NULL, NULL);
+	
+	return TRUE;
+}
+
+
 /** Internal HAL initialization function
  *
  * @return			The LibHalContext of the HAL connection or
  *				NULL on error.
  */
 static LibHalContext *
-gvm_do_hal_init (void)
+gvm_hal_init (void)
 {
 	LibHalContext *ctx;
 	DBusError error;
 	char **devices;
 	int nr;
 	
+	if (!gvm_dbus_init ())
+		return NULL;
+	
 	if (!(ctx = libhal_ctx_new ())) {
-		warn ("failed to initialize HAL!");
+		warn ("failed to create a HAL context!");
 		return NULL;
 	}
 	
-	dbus_error_init (&error);
-	if (!hal_mainloop_integration (ctx, &error)) {
-		warn ("hal_initialize failed: %s", error.message);
-		dbus_error_free (&error);
-		return NULL;
-	}
+	libhal_ctx_set_dbus_connection (ctx, dbus_connection);
 	
 	libhal_ctx_set_device_added (ctx, hal_device_added);
 	libhal_ctx_set_device_removed (ctx, hal_device_removed);
@@ -2577,6 +2587,7 @@ gvm_do_hal_init (void)
 	libhal_ctx_set_device_property_modified (ctx, hal_property_modified);
 	libhal_ctx_set_device_condition (ctx, hal_device_condition);
 	
+	dbus_error_init (&error);
 	if (!libhal_device_property_watch_all (ctx, &error)) {
 		warn ("failed to watch all HAL properties!: %s", error.message);
 		dbus_error_free (&error);
@@ -2585,7 +2596,7 @@ gvm_do_hal_init (void)
 	}
 	
 	if (!libhal_ctx_init (ctx, &error)) {
-		warn ("hal_initialize failed: %s", error.message);
+		warn ("libhal_ctx_init failed: %s", error.message);
 		dbus_error_free (&error);
 		libhal_ctx_free (ctx);
 		return NULL;
@@ -2666,7 +2677,7 @@ mount_all (LibHalContext *ctx)
 		if (!libhal_device_property_exists (ctx, udi, "volume.fsusage", NULL))
 			continue;
 		prop = libhal_device_get_property_string (ctx, udi, "volume.fsusage", NULL);
-		if (!prop || strcmp (prop, "filesystem") != 0) {
+		if (!prop || ((strcmp (prop, "filesystem") != 0) && (strcmp (prop, "crypto") != 0))) {
 			libhal_free_string (prop);
 			continue;
 		}
@@ -2683,7 +2694,7 @@ mount_all (LibHalContext *ctx)
 			 && libhal_device_get_property_bool (ctx, drive, "storage.removable", NULL))
 			mount = config.automount_media;
 		else
-			mount = gvm_automount_enabled (udi);
+			mount = !libhal_device_get_property_bool (ctx, udi, "volume.ignore", NULL);
 		libhal_free_string (drive);
 		
 		if (!mount)
@@ -2751,7 +2762,7 @@ sigterm_iochn_data (GIOChannel *source,
 	GError *err = NULL;
 	gchar data[1];
 	gsize bytes_read;
-
+	
 	/* Empty the pipe */
 	if (G_IO_STATUS_NORMAL != g_io_channel_read_chars (source, data, 1, &bytes_read, &err)) {
 		warn ("Error emptying callout notify pipe: %s", err->message);
@@ -2763,13 +2774,6 @@ sigterm_iochn_data (GIOChannel *source,
 	
 	unmount_all ();
 	
-#ifdef ENABLE_NOTIFY
-	g_source_remove (statfs_id);
-	statfs_id = 0;
-	
-	g_hash_table_destroy (statfs_mounts);
-#endif
-
 	gtk_main_quit ();
 	
  out:
@@ -2777,6 +2781,7 @@ sigterm_iochn_data (GIOChannel *source,
 }
 
 
+#ifdef ENABLE_NOTIFY
 static void
 statfs_mount_info_add (const char *udi)
 {
@@ -2785,7 +2790,7 @@ statfs_mount_info_add (const char *udi)
 	info = g_new0 (statfs_mount_info, 1);
 	info->udi = g_strdup (udi);
 	info->notified = FALSE;
-	info->last_pct = 0.0;
+	info->last_notified = 0.0;
 	
 	g_hash_table_insert (statfs_mounts, info->udi, info);
 	
@@ -2802,7 +2807,6 @@ statfs_mount_info_free (statfs_mount_info *info)
 static gboolean
 gvm_statfs_check_space (const char *udi, statfs_mount_info *info, gpointer user_data)
 {
-#ifdef ENABLE_NOTIFY
 	char *mount_point = NULL;
 	NotifyNotification *n;
 	struct statfs buf;
@@ -2814,64 +2818,54 @@ gvm_statfs_check_space (const char *udi, statfs_mount_info *info, gpointer user_
 		double free_space;
 		
 		free_space = (double) buf.f_bavail / (double) buf.f_blocks;
-		
-		if ((free_space - info->last_pct) > config.percent_used)
+
+		/*
+		   If we continue using up more and more space or if we pass
+		   the threshold value while running, we want to notify again
+		*/
+		if ((info->last_notified - free_space) > config.percent_used
+		    || (free_space < config.percent_threshold && info->last_notified > config.percent_threshold))
 			info->notified = FALSE;
-		else if ((free_space - info->last_pct) < config.percent_freed)
-			info->notified = TRUE;
 		
-		if (!info->notified && free_space < config.percent_threshold) {
-			char *low_disk_msg = NULL;
-			char *label = NULL;
+		if (!info->notified && (free_space < config.percent_threshold)) {
+			char *disk, *label, *msg;
 			int free_pct;
 			
 			label = libhal_device_get_property_string (hal_ctx, udi, "volume.label", NULL);
-			if (!label || label[0] == '\0') {
-				libhal_free_string (label);
-				label = g_strdup (mount_point);
-			}
-			
-			free_pct = 100 - (free_space * 100);
-			low_disk_msg = g_strdup_printf (_("%d%% of the disk space on %s is in use"), free_pct, label);
-			
-			n = notify_notification_new ("Low Disk Space", low_disk_msg, "gnome-dev-harddisk", NULL);
-			notify_notification_set_urgency (n, NOTIFY_URGENCY_CRITICAL);
-			notify_notification_show (n, NULL);
-			
-			g_object_unref (n);
-			
-			libhal_free_string (low_disk_msg);
+			if (!label || label[0] == '\0')
+				disk = g_strdup (mount_point);
+			else
+				disk = g_strdup (label);
 			libhal_free_string (label);
 			
+			free_pct = 100 - (free_space * 100);
+			msg = g_strdup_printf (_("%d%% of the disk space on %s is in use"), free_pct, disk);
+			g_free (disk);
+			
+			n = notify_notification_new ("Low Disk Space", msg, "gnome-dev-harddisk", NULL);
+			notify_notification_set_urgency (n, NOTIFY_URGENCY_CRITICAL);
+			notify_notification_show (n, NULL);
+			g_object_unref (n);
+			g_free (msg);
+			
 			info->notified = TRUE;
+			info->last_notified = free_space;
 		}
-		
-		info->last_pct = free_space;
 	}
 	
 	libhal_free_string (mount_point);
 	
 	return TRUE;
-#else
-	udi = udi;
-	info = info;
-	user_data = user_data;
-	
-	return FALSE;
-#endif
 }
 
 static gboolean
 gvm_statfs_timeout (gpointer data)
 {
-#ifdef ENABLE_NOTIFY
 	g_hash_table_foreach (statfs_mounts, (GHFunc) gvm_statfs_check_space, NULL);
 	
 	return TRUE;
-#else
-	return FALSE;
-#endif
 }
+#endif /* ENABLE_NOTIFY */
 
 static void
 gvm_die (GnomeClient *client __attribute__((__unused__)),
@@ -2880,13 +2874,6 @@ gvm_die (GnomeClient *client __attribute__((__unused__)),
 	dbg ("Received 'die', initiating shutdown\n");
 	
 	unmount_all ();
-
-#ifdef ENABLE_NOTIFY
-	g_source_remove (statfs_id);
-	statfs_id = 0;
-	
-	g_hash_table_destroy (statfs_mounts);
-#endif
 	
 	gtk_main_quit ();
 }
@@ -2896,6 +2883,7 @@ gvm_die (GnomeClient *client __attribute__((__unused__)),
 static gboolean
 gvm_user_at_console (void)
 {
+#ifdef ENABLE_MULTIUSER
 	static unsigned int exists = 0;
 	char *path;
 	
@@ -2906,14 +2894,23 @@ gvm_user_at_console (void)
 	}
 	
 	return (exists & 0x1);
+#else
+	/* for the non-multiuser configuration, we assume the user is at the console no matter what */
+	return TRUE;
+#endif
 }
 
 /* checks that the user's session is the active session at the console */
 static gboolean
 gvm_user_active_at_console (void)
 {
+#ifdef ENABLE_MULTIUSER
 	/* FIXME: it is currently impossible to get this information */
 	return TRUE;
+#else
+	/* for the non-multiuser configuration, we assume we're always on the active console */
+	return TRUE;
+#endif
 }
 
 
@@ -2989,7 +2986,7 @@ main (int argc, char **argv)
 	
 	g_signal_connect (client, "die", G_CALLBACK (gvm_die), NULL);
 	
-	if (!(hal_ctx = gvm_do_hal_init ()))
+	if (!(hal_ctx = gvm_hal_init ()))
 		return 1;
 	
 	gvm_init_config ();
@@ -3021,14 +3018,18 @@ main (int argc, char **argv)
 #endif
 
 	mount_all (hal_ctx);
-
+	
+#ifdef ENABLE_NOTIFY
 	/* Add a timeout for our statfs checking */
 	statfs_id = g_timeout_add (15000, (GSourceFunc) gvm_statfs_timeout, NULL);
+#endif
 	
 	gtk_main ();
 	
 #ifdef ENABLE_NOTIFY
 	g_hash_table_destroy (statfs_mounts);
+	g_source_remove (statfs_id);
+	statfs_id = 0;
 #endif
 	
 	g_hash_table_destroy (mount_table);
