@@ -13,6 +13,10 @@
 #include "config.h"
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 #include <gnome.h>
 #include <gdk/gdkx.h>
 #include <glib/gi18n.h>
@@ -63,7 +67,6 @@ enum {
 	AUTOVIDEOCAM,
 	AUTOVIDEOCAM_COMMAND,
 	AUTORUN_PATH,
-	EJECT_COMMAND,
 };
 
 static struct {
@@ -104,8 +107,7 @@ static struct {
 	{ GCONF_ROOT "autovideocam",              TYPE_BOOL,   NULL, TRUE  },
 	{ GCONF_ROOT "autovideocam_command",      TYPE_STRING, NULL, FALSE },
 	/* The following entries do not yet have a UI */
-	/*{ GCONF_ROOT "autorun_path",         TYPE_STRING, NULL, FALSE },
-	  { GCONF_ROOT "eject_command",        TYPE_STRING, NULL, FALSE },*/
+	/*{ GCONF_ROOT "autorun_path",         TYPE_STRING, NULL, FALSE },*/
 };
 
 static GHashTable *gvm_settings_hash = NULL;
@@ -231,22 +233,64 @@ close_cb (GtkWidget *dialog,
  * a given GtkImage.
  */
 static void
-set_icon (GtkImage *image, GnomeIconTheme *theme, const char *name)
+set_icon (GtkImage *image, GtkIconTheme *theme, const char *name)
 {
-	char *file;
+	GdkPixbuf *pixbuf;
+	struct stat st;
 	
-	file = gnome_icon_theme_lookup_icon (theme, name, 48, NULL, NULL);
-	if (file) {
-		gtk_image_set_from_file (image, file);
-		g_free (file);
+	if (*name == '/') {
+		if (stat (name, &st) == 0 && S_ISREG (st.st_mode)) {
+			gtk_image_set_from_file (image, name);
+			return;
+		} else
+			name = "gnome-dev-broken-image";
+	}
+	
+	if ((pixbuf = gtk_icon_theme_load_icon (theme, name, 48, 0, NULL))) {
+		gtk_image_set_from_pixbuf (image, pixbuf);
+		g_object_unref (pixbuf);
+	}
+}
+
+static struct {
+	const char *name;
+	const char *icon;
+} icons[] = {
+	{ "audio_cd_image",         "gnome-dev-cdrom-audio"   },
+	{ "blank_cd_image",         "gnome-dev-disc-cdr"      },
+	{ "digital_camera_image",   "applets-screenshooter"   },
+	{ "dvd_video_image",        "gnome-dev-dvd"           },
+	{ "ipod_image",             "gnome-dev-ipod"          },
+	{ "keyboard_image",         "gnome-dev-keyboard"      },
+	{ "mouse_image",            "gnome-dev-mouse-optical" },
+	{ "palm_image",             "palm-pilot"              },
+	/* FIXME: need a new icon */
+	{ "pocketpc_image",         "palm-pilot"              },
+	{ "printer_image",          "gnome-dev-printer"       },
+	{ "removable_drives_image", "gnome-dev-removable"     },
+	{ "scanner_image",          "xsane"                   },
+	/* FIXME: need better icon */
+	{ "tablet_image",           ICONSDIR "/gvm-dev-tablet.png" },
+	/* FIXME: need better icon */
+	{ "videocam_image",         "gnome-mime-video"        }
+};
+
+static void
+theme_changed_cb (GtkIconTheme *theme, GladeXML *xml)
+{
+	GtkWidget *icon;
+	size_t i;
+	
+	for (i = 0; i < G_N_ELEMENTS (icons); i++) {
+		icon = glade_xml_get_widget (xml, icons[i].name);
+		set_icon (GTK_IMAGE (icon), theme, icons[i].icon);
 	}
 }
 
 static void
 show_props (void)
 {
-	GnomeIconTheme *icon_theme;
-	GtkWidget *icon;
+	GtkIconTheme *theme;
 	char *filename;
 	size_t i;
 	
@@ -273,43 +317,12 @@ show_props (void)
 	props = glade_xml_get_widget (xml, "dialog1");
 	g_signal_connect (props, "response", G_CALLBACK (close_cb), NULL);
 	
-	icon_theme = gnome_icon_theme_new ();
-	
-	icon = glade_xml_get_widget (xml, "removable_drives_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-removable");
-	icon = glade_xml_get_widget (xml, "audio_cd_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-cdrom-audio");
-	icon = glade_xml_get_widget (xml, "blank_cd_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-disc-cdr");
-	icon = glade_xml_get_widget (xml, "dvd_video_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-dvd");
-	icon = glade_xml_get_widget (xml, "digital_camera_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "applets-screenshooter");
-	icon = glade_xml_get_widget (xml, "ipod_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-ipod");
-	icon = glade_xml_get_widget (xml, "printer_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-printer");
-	icon = glade_xml_get_widget (xml, "scanner_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "xsane");
-	icon = glade_xml_get_widget (xml, "keyboard_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-keyboard");
-	icon = glade_xml_get_widget (xml, "mouse_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-mouse-optical");
-	/* FIXME: need better icon */
-	icon = glade_xml_get_widget (xml, "tablet_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-mouse-optical");
-	/* FIXME: need better icon */
-	icon = glade_xml_get_widget (xml, "videocam_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-mime-video");
-	icon = glade_xml_get_widget (xml, "palm_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "palm-pilot");
-	icon = glade_xml_get_widget (xml, "pocketpc_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "palm-pilot");
+	theme = gtk_icon_theme_get_default ();
+	g_signal_connect (theme, "changed", G_CALLBACK (theme_changed_cb), xml);
+	theme_changed_cb (theme, xml);
 	
 	gtk_window_set_default_icon_name ("gnome-dev-cdrom");
 	
-	g_object_unref (icon_theme);
-	
 	gvm_settings_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	for (i = 0; i < G_N_ELEMENTS (gvm_settings); i++) {
 		const char *name;
@@ -466,7 +479,7 @@ int
 main (int argc, char **argv)
 {
 	gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
-			    GNOME_PARAM_NONE);
+			    GNOME_PARAM_NONE, GNOME_PARAM_NONE);
 	
 	glade_gnome_init ();
 	
